@@ -687,6 +687,9 @@ panel_menu_button_load (const char  *menu_path,
 
 	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("_Help"), NULL);
 
+        if (panel_is_program_in_path ("gmenu-simple-editor"))
+		panel_applet_add_callback (info, "edit", NULL, _("_Edit Menus"), NULL);
+
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
 
@@ -699,8 +702,8 @@ panel_menu_button_load (const char  *menu_path,
 static char *
 panel_menu_button_get_icon (PanelMenuButton *button)
 {
-	MenuTreeDirectory *directory;
-        char              *retval;
+	GMenuTreeDirectory *directory;
+        char               *retval;
 
 	retval = NULL;
 
@@ -716,21 +719,21 @@ panel_menu_button_get_icon (PanelMenuButton *button)
 					       "panel-menu-tree-directory");
 
 		if (!directory) {
-			MenuTree *tree;
+			GMenuTree *tree;
 
 			if ((tree = g_object_get_data (G_OBJECT (button->priv->menu),
 						       "panel-menu-tree"))) {
-				directory = menu_tree_get_directory_from_path (tree,
-									       button->priv->menu_path);
+				directory = gmenu_tree_get_directory_from_path (tree,
+										button->priv->menu_path);
 				g_object_set_data_full (G_OBJECT (button->priv->menu),
 							"panel-menu-tree-directory",
 							directory,
-							(GDestroyNotify) menu_tree_directory_unref);
+							(GDestroyNotify) gmenu_tree_item_unref);
 			}
 		}
 
 		if (directory)
-			retval = g_strdup (menu_tree_directory_get_icon (directory));
+			retval = g_strdup (gmenu_tree_directory_get_icon (directory));
 	}
 
 	if (!retval)
@@ -954,7 +957,6 @@ panel_menu_button_create (PanelToplevel *toplevel,
 
 	if (filename && !scheme) {
 		g_warning ("Failed to find menu scheme for %s\n", filename);
-		g_free (id);
 		return FALSE;
 	}
 
@@ -974,8 +976,8 @@ panel_menu_button_create (PanelToplevel *toplevel,
 		gconf_client_set_string (client, key, tooltip, NULL);
 	}
 
+	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
-	g_free (id);
 
 	return TRUE;
 }
@@ -989,12 +991,27 @@ panel_menu_button_invoke_menu (PanelMenuButton *button,
 	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
 	g_return_if_fail (callback_name != NULL);
 
-	if (strcmp (callback_name, "help"))
-		return;
-
 	screen = gtk_widget_get_screen (GTK_WIDGET (button));
 
-	panel_show_help (screen, "user-guide.xml", "gospanel-37");
+	if (!strcmp (callback_name, "help")) {
+		panel_show_help (screen, "user-guide.xml", "gospanel-37");
+
+	} else if (!strcmp (callback_name, "edit")) {
+                GError *error = NULL;
+                char   *argv [2] = {"gmenu-simple-editor", NULL};
+
+                if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
+                                          G_SPAWN_SEARCH_PATH,
+                                          NULL, NULL, NULL, &error)) {
+                        panel_error_dialog (screen,
+                                            "cannot_exec_gmenu-simple-editor", TRUE,
+                                            _("Cannot execute '%s'"),
+                                            "%s",
+                                            "gmenu-simple-editor",
+                                            error->message);
+                        g_error_free (error);
+                }
+	}
 }
 
 void
